@@ -2,17 +2,18 @@
 import anydbm
 import cPickle
 import logging
-import os.path
+import os
 import time
 
 import beaker.util as util
 from beaker.exceptions import CreationAbortedError, MissingCacheParameter
 from beaker.synchronization import _threading, file_synchronizer, \
-     mutex_synchronizer, NameLock
+     mutex_synchronizer, NameLock, null_synchronizer
 
 __all__ = ['Value', 'Container', 'ContainerContext',
            'MemoryContainer', 'DBMContainer', 'NamespaceManager',
            'MemoryNamespaceManager', 'DBMNamespaceManager', 'FileContainer',
+           'OpenResourceNamespaceManager',
            'FileNamespaceManager', 'CreationAbortedError']
 
 
@@ -46,27 +47,33 @@ class NamespaceManager(object):
     implementation.
 
     """
+    
+    @classmethod
+    def _init_dependencies(cls):
+        pass
+        
     def __init__(self, namespace):
+        self._init_dependencies()
         self.namespace = namespace
-        self.openers = 0
-        self.mutex = _threading.Lock()
-        self.access_lock = self.get_access_lock()
         
     def get_creation_lock(self, key):
         raise NotImplementedError()
 
-    def get_access_lock(self):
-        raise NotImplementedError()
-        
-    def do_open(self, flags): 
-        raise NotImplementedError()
-        
-    def do_close(self): 
-        raise NotImplementedError()
-
     def do_remove(self):
         raise NotImplementedError()
 
+    def acquire_read_lock(self):
+        pass
+
+    def release_read_lock(self):
+        pass
+
+    def acquire_write_lock(self, wait=True):
+        return True
+
+    def release_write_lock(self):
+        pass
+
     def has_key(self, key):
         return self.__contains__(key)
 
@@ -94,6 +101,30 @@ class NamespaceManager(object):
     
     def keys(self):
         raise NotImplementedError()
+    
+    def remove(self):
+        self.do_remove()
+        
+
+class OpenResourceNamespaceManager(NamespaceManager):
+    """A NamespaceManager where read/write operations require opening/
+    closing of a resource which is possibly mutexed.
+    
+    """
+    def __init__(self, namespace):
+        NamespaceManager.__init__(self, namespace)
+        self.access_lock = self.get_access_lock()
+        self.openers = 0
+        self.mutex = _threading.Lock()
+
+    def get_access_lock(self):
+        raise NotImplementedError()
+
+    def do_open(self, flags): 
+        raise NotImplementedError()
+
+    def do_close(self): 
+        raise NotImplementedError()
 
     def acquire_read_lock(self): 
         self.access_lock.acquire_read_lock()
@@ -160,7 +191,6 @@ class NamespaceManager(object):
         finally:
             self.access_lock.release_write_lock()
 
-
 class Value(object):
     __slots__ = 'key', 'createfunc', 'expiretime', 'expire_argument', 'starttime', 'storedtime',\
                 'namespace'
@@ -193,29 +223,26 @@ class Value(object):
         try:    
             has_value = self.namespace.has_key(self.key)
             if has_value:
-                self.storedtime, self.expiretime, value = self.namespace[self.key]
-                return not self._is_expired()
-            else:
-                return False
+                try:
+                    stored, expired, value = self._get_value()
+                    return not self._is_expired(stored, expired)
+                except KeyError:
+                    pass
+            return False
         finally:
             self.namespace.release_read_lock()
 
-    def _is_expired(self):
-        """Return true if this container's value is expired.
-        
-        Note that this method is only correct if has_current_value()
-        or get_value() have been called already.
-        
-        """
+    def _is_expired(self, storedtime, expiretime):
+        """Return true if this container's value is expired."""
         return (
             (
                 self.starttime is not None and
-                self.storedtime < self.starttime
+                storedtime < self.starttime
             )
             or
             (
-                self.expiretime is not None and
-                time.time() >= self.expiretime + self.storedtime
+                expiretime is not None and
+                time.time() >= expiretime + storedtime
             )
         )
 
@@ -224,10 +251,14 @@ class Value(object):
         try:
             has_value = self.has_value()
             if has_value:
-                value = self.__get_value()
-                if not self._is_expired():
-                    return value
-
+                try:
+                    stored, expired, value = self._get_value()
+                    if not self._is_expired(stored, expired):
+                        return value
+                except KeyError:
+                    # guard against un-mutexed backends raising KeyError
+                    has_value = False
+                    
             if not self.createfunc:
                 raise KeyError(self.key)
         finally:
@@ -253,9 +284,13 @@ class Value(object):
             self.namespace.acquire_read_lock()
             try:
                 if self.has_value():
-                    value = self.__get_value()
-                    if not self._is_expired():
-                        return value
+                    try:
+                        stored, expired, value = self._get_value()
+                        if not self._is_expired(stored, expired):
+                            return value
+                    except KeyError:
+                        # guard against un-mutexed backends raising KeyError
+                        pass
             finally:
                 self.namespace.release_read_lock()
 
@@ -267,28 +302,34 @@ class Value(object):
             creation_lock.release()
             debug("released create lock")
 
-    def __get_value(self):
+    def _get_value(self):
         value = self.namespace[self.key]
         try:
-            self.storedtime, self.expiretime, value = value
+            stored, expired, value = value
         except ValueError:
             if not len(value) == 2:
                 raise
             # Old format: upgrade
-            self.storedtime, value = value
-            self.expiretime = self.expire_argument = None
-            debug("get_value upgrading time %r expire time %r", self.storedtime, self.expire_argument)
+            stored, value = value
+            expired = self.expire_argument
+            debug("get_value upgrading time %r expire time %r", stored, self.expire_argument)
             self.namespace.release_read_lock()
-            self.set_value(value)
+            self.set_value(value, stored)
             self.namespace.acquire_read_lock()
-        return value
+        except TypeError:
+            # occurs when the value is None.  memcached 
+            # may yank the rug from under us in which case 
+            # that's the result
+            raise KeyError(self.key)            
+        return stored, expired, value
 
-    def set_value(self, value):
+    def set_value(self, value, storedtime=None):
         self.namespace.acquire_write_lock()
         try:
-            self.storedtime = time.time()
-            debug("set_value stored time %r expire time %r", self.storedtime, self.expire_argument)
-            self.namespace.set_value(self.key, (self.storedtime, self.expire_argument, value), expiretime=self.expire_argument)
+            if storedtime is None:
+                storedtime = time.time()
+            debug("set_value stored time %r expire time %r", storedtime, self.expire_argument)
+            self.namespace.set_value(self.key, (storedtime, self.expire_argument, value))
         finally:
             self.namespace.release_write_lock()
 
@@ -297,36 +338,46 @@ class Value(object):
         try:
             debug("clear_value")
             if self.namespace.has_key(self.key):
-                del self.namespace[self.key]
+                try:
+                    del self.namespace[self.key]
+                except KeyError:
+                    # guard against un-mutexed backends raising KeyError
+                    pass
             self.storedtime = -1
         finally:
             self.namespace.release_write_lock()
 
-
-class MemoryNamespaceManager(NamespaceManager):
-    namespaces = util.SyncDict()
-
-    def __init__(self, namespace, **kwargs):
-        NamespaceManager.__init__(self, namespace)
-        self.dictionary = MemoryNamespaceManager.namespaces.get(self.namespace,
-                                                                dict)
+class AbstractDictionaryNSManager(NamespaceManager):
+    """A subclassable NamespaceManager that places data in a dictionary.
     
-    def get_access_lock(self):
-        return mutex_synchronizer(
-            identifier="memorycontainer/namespacelock/%s" % self.namespace)
+    Subclasses should provide a "dictionary" attribute or descriptor
+    which returns a dict-like object.   The dictionary will store keys
+    that are local to the "namespace" attribute of this manager, so
+    ensure that the dictionary will not be used by any other namespace.
+
+    e.g.::
+    
+        import collections
+        cached_data = collections.defaultdict(dict)
         
+        class MyDictionaryManager(AbstractDictionaryNSManager):
+            def __init__(self, namespace):
+                AbstractDictionaryNSManager.__init__(self, namespace)
+                self.dictionary = cached_data[self.namespace]
+                
+    The above stores data in a global dictionary called "cached_data",
+    which is structured as a dictionary of dictionaries, keyed
+    first on namespace name to a sub-dictionary, then on actual
+    cache key to value.
+    
+    """
+    
     def get_creation_lock(self, key):
         return NameLock(
-            identifier="memorycontainer/funclock/%s/%s" % (self.namespace, key),
+            identifier="memorynamespace/funclock/%s/%s" % (self.namespace, key),
             reentrant=True
         )
-        
-    def open(self, *args, **kwargs):
-        pass
-        
-    def close(self, *args, **kwargs):
-        pass
-    
+
     def __getitem__(self, key): 
         return self.dictionary[key]
 
@@ -347,9 +398,16 @@ class MemoryNamespaceManager(NamespaceManager):
         
     def keys(self):
         return self.dictionary.keys()
+    
+class MemoryNamespaceManager(AbstractDictionaryNSManager):
+    namespaces = util.SyncDict()
 
+    def __init__(self, namespace, **kwargs):
+        AbstractDictionaryNSManager.__init__(self, namespace)
+        self.dictionary = MemoryNamespaceManager.namespaces.get(self.namespace,
+                                                                dict)
 
-class DBMNamespaceManager(NamespaceManager):
+class DBMNamespaceManager(OpenResourceNamespaceManager):
     def __init__(self, namespace, dbmmodule=None, data_dir=None, 
             dbm_dir=None, lock_dir=None, digest_filenames=True, **kwargs):
         self.digest_filenames = digest_filenames
@@ -373,7 +431,7 @@ class DBMNamespaceManager(NamespaceManager):
         self.dbmmodule = dbmmodule or anydbm
 
         self.dbm = None
-        NamespaceManager.__init__(self, namespace)
+        OpenResourceNamespaceManager.__init__(self, namespace)
 
         self.file = util.encoded_path(root= self.dbm_dir,
                                       identifiers=[self.namespace],
@@ -451,7 +509,7 @@ class DBMNamespaceManager(NamespaceManager):
         return self.dbm.keys()
 
 
-class FileNamespaceManager(NamespaceManager):
+class FileNamespaceManager(OpenResourceNamespaceManager):
     def __init__(self, namespace, data_dir=None, file_dir=None, lock_dir=None,
                  digest_filenames=True, **kwargs):
         self.digest_filenames = digest_filenames
@@ -471,7 +529,7 @@ class FileNamespaceManager(NamespaceManager):
         else:
             self.lock_dir = data_dir + "/container_file_lock"
         util.verify_directory(self.lock_dir)
-        NamespaceManager.__init__(self, namespace)
+        OpenResourceNamespaceManager.__init__(self, namespace)
 
         self.file = util.encoded_path(root=self.file_dir, 
                                       identifiers=[self.namespace],
@@ -499,7 +557,7 @@ class FileNamespaceManager(NamespaceManager):
             fh = open(self.file, 'rb')
             try:
                 self.hash = cPickle.load(fh)
-            except (IOError, OSError, EOFError, cPickle.PickleError):
+            except (IOError, OSError, EOFError, cPickle.PickleError, ValueError):
                 pass
             fh.close()
 
@@ -515,7 +573,12 @@ class FileNamespaceManager(NamespaceManager):
         self.flags = None
                 
     def do_remove(self):
-        os.remove(self.file)
+        try:
+            os.remove(self.file)
+        except OSError, err:
+            # for instance, because we haven't yet used this cache,
+            # but client code has asked for a clear() operation...
+            pass
         self.hash = {}
         
     def __getitem__(self, key): 
